<?php
namespace Teknisa\PLA\Services;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
class Alergenico {
    
    protected $crudService;
    protected $imgPathURL;
    protected $dataSourceName = 'alergenico';
    
    public function __construct($crudService,$imgPathURL) {
	
		$this->crudService = $crudService;
		$this->imgPathURL = $imgPathURL;
		
	}

    const ICONS = array(
        "SALSAO" => "celery.png",
        "CRUSTACEO" => "crustaceans.png",
        "OVO" => "egg.png",
        "PEIXE" => "fish.png",
        "GLUTEN" => "gluten.png",
        "TREMOCO" => "lupins.png",
        "LEITE" => "milk.png",
        "NOZ" => "nut.png",
        "MOSTARDA" => "mustard.png",
        "AMENDOIM" => "peanut.png",
        "SESAMO" => "sesame.png",
        "MARISCO" => "shellfish.png",
        "SOJA" => "soy.png",
        "SULFITO" => "sulfites.png"
    );

    public function save(DTO\Request\Row $request, DTO\Response $response) {
       $dataSet = new DataSet($this->dataSourceName, array($request->getRow()));
	   $newRequest = new DTO\Request\DataSet($dataSet, '', '', '');
	   $rowsToSave = $this->crudService->getRowsToSave($newRequest, $this->dataSourceName);
	   $this->crudService->save($this->dataSourceName, $rowsToSave);
	   $response->addDataSet(new DataSet("columnsToReturn", $this->crudService->getColumnsToReturn()));
    }
    
    private function handleException( $message ){
        preg_match("/ORA-[0-9]{5}/", $message, $matches);
        if(!empty($matches)){
            list($exceptionCode) = $matches; 
            switch( $exceptionCode ) {
                case "ORA-00001": return "ERR_MSG_RECORD_ALREADY_EXISTS";
                case "ORA-02292": return "ERR_MSG_RECORD_PRODUCT";
                default: return $message;
            }
        } else {
            return $message;
        }
    }
    
    public function remove(DTO\Request\Row $request, DTO\Response $response) {
        try {
            $dataSet = new DataSet($this->dataSourceName, array($request->getRow()));
    	    $newRequest = new DTO\Request\DataSet($dataSet, '', '', '');
    	    $rowsToDelete = $this->crudService->getRowsToDelete($newRequest, $this->dataSourceName);
    	    $dataSet = new DataSet($this->dataSourceName, (array)$rowsToDelete);
    	    $this->crudService->delete($response, $dataSet);
        } catch(\Exception $e) {
            $message = $this->handleException($e->getMessage());
            $response->setError(new Error($message, $e->getCode()));    
        }
    }

	public function allergenicIcons(DTO\Request $request, DTO\Response $response){
        $basePath = __DIR__."/../../../../reports/src/assets/img/";
        $dataSet = [];
        $count = 0;
        foreach(self::ICONS as $iconKey => $fileName){
            $filePath = $basePath . $fileName;
            $type = pathinfo($filePath, PATHINFO_EXTENSION);
            $data = file_get_contents($filePath);
            $b64 = 'data:image/'. $type . ';base64,' . base64_encode($data);
            $dataSet[] = [
                "ID" => ++$count,
                "IDICONE" => $iconKey,
                "TYPE" => "BASE64",
                "ICON" => $b64,
                "DSICONE" => $iconKey,
                "NRIMAGEMALERGENICO" => null
            ];
        }
        
        $conditions = [];
		$imagemAlergenico = $this->crudService->getFilteredDataSet("imagemalergenico", $conditions)->getRows();
        
        foreach ( $imagemAlergenico as $index => $row ) {
            if ($row["DSIMAGEM"] !== null) {
                    $dataSet[] = [
                        "ID"      => ++$count,
                        "TYPE"    => "URL",
                        "IDICONE" => $row['NRIMAGEMALERGENICO'],
                        "ICON"    => $this->imgPathURL.$row["DSIMAGEM"],
                        "DSICONE" => null,
                        "NRIMAGEMALERGENICO" => $row['NRIMAGEMALERGENICO']
                    ];
            }
        }
        
        $response->addDataSet(new DataSet("allergenicIcons", $dataSet));
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $dataSourceName = "alergenico";
        $filterCriteria = $request->getFilterCriteria();
        $filterCriteria->setDataSourceName($dataSourceName);
        $this->crudService->find($response, $filterCriteria);
        list($allergenicsData) = $response->getDataSets();
        $dataSet = $allergenicsData->getRows();
        $newRows = [];
        foreach( $dataSet as $row ) {
            $row['IDICONE'] = $row['DSICONE'];
            if($row['NRIMAGEMALERGENICO']){
                $row['IDICONE'] = $row['NRIMAGEMALERGENICO'];
            }
            $newRows[] = $row;
        }
        $newDataSet = new DataSet($dataSourceName, $newRows);
        $response->setDataSets([$newDataSet]);
    }
    
}